/*
 * MobileEdit v. 1.4.1, editor for mobile phones with Java support
 * Copyright (C) 2002 Jens Lindemann
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Soft-
 * ware Foundation; either version 2 of this license, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTIBILITY or
 * FITTNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.rms.*;
import javax.microedition.io.*;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

public class MobileEdit extends MIDlet implements CommandListener {
    
    private Command newLineCommand; // The new line command
    private Command exitCommand; // The exit command
    private Command saveCommand; // The save command
    private Command loadCommand; // The load command
    private Command titleOKCommand; // The title OK command... :-P
    private Display display;    // The display for this MIDlet
    private TextBox t = new TextBox("Note", "", 2000, 0);
    private RecordStore recordStore;
    private String noteContent;
    private String noteTitle;
    
    public MobileEdit() {
        display = Display.getDisplay(this);
        
        newLineCommand = new Command("New Line", Command.SCREEN, 2);
        exitCommand = new Command("Exit", Command.EXIT, 3);
        saveCommand = new Command("Save", Command.SCREEN, 4);
        loadCommand = new Command("Load", Command.SCREEN, 5);
        
        titleOKCommand = new Command("OK", Command.OK, 101);
    }
    
    /**
     * Start up the MIDlet by creating the TextBox and associating
     * the exit command and listener.
     */
    public void startApp() {        
        t.addCommand(newLineCommand);
        t.addCommand(exitCommand);
        t.addCommand(saveCommand);
        t.addCommand(loadCommand);
        t.setCommandListener(this);
        
        display.setCurrent(t);
    }
    
    /**
     * Pause is a no-op since there are no background activities or
     * record stores that need to be closed.
     */
    public void pauseApp() {
    }
    
    /**
     * Destroy must cleanup everything not handled by the garbage collector.
     * In this case there is nothing to cleanup.
     */
    public void destroyApp(boolean unconditional) {
    }
    
    /**
     * SaveNote saves your note.
     */
    public void saveNote() {
        noteContent = t.getString();
            
        t.setTitle("Enter title");
        t.setString("");
            
        t.addCommand(titleOKCommand);
        t.removeCommand(newLineCommand);
        t.removeCommand(exitCommand);
        t.removeCommand(saveCommand);
        t.removeCommand(loadCommand);
    }
    
    public void saveNote2() {
        try { 
            noteTitle = t.getString();
            
            t.removeCommand(titleOKCommand);
            t.addCommand(newLineCommand);
            t.addCommand(exitCommand);
            t.addCommand(saveCommand);
            t.addCommand(loadCommand);
            
            t.setTitle(noteTitle);
            t.setString(noteContent);
                              
            RecordStore recordStore = RecordStore.openRecordStore("note", true);
            
            // Create output streams.
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            DataOutputStream os = new DataOutputStream(baos);
            
            // Write the values to be saved to the output stream
            os.writeUTF(noteTitle);
            os.writeUTF(noteContent);
            
            // Get the byte array with the values that should be saved
            byte[] data = baos.toByteArray();
        
            // Check if the records already exist, if not create them
            if (recordStore.getNextRecordID() == 1) {
                recordStore.addRecord(data, 0, data.length);
            } else {
                recordStore.setRecord(1, data, 0, data.length);
            }
            
            recordStore.closeRecordStore();
        } catch (RecordStoreException rse) {
            Alert rseAlert = new Alert("ERROR", "RecordStoreException", null, 
                                        AlertType.ERROR);
            rseAlert.setTimeout(Alert.FOREVER);
            Display.getDisplay(this).setCurrent(rseAlert);
        } catch (IOException ioe) {
              Alert ioeAlert = new Alert("ERROR", "I/O Exception", null, 
                                         AlertType.ERROR);
              ioeAlert.setTimeout(Alert.FOREVER);
              Display.getDisplay(this).setCurrent(ioeAlert);
        }
    }
    
    /**
     * LoadNote loads your note.
     */
    public void loadNote() {
        try {
            // Open record store
            RecordStore recordStore = RecordStore.openRecordStore("note", true);
            // Read the data from the record store
            byte[] data = recordStore.getRecord(1);
            DataInputStream is = new DataInputStream(new ByteArrayInputStream(data));
            
            // Read note title and contents from the record store
            noteTitle = is.readUTF();
            noteContent = is.readUTF();
            
            // Close the input stream
            is.close();
            // Close record store
            recordStore.closeRecordStore();
            
            // Transfer the note content and title to the TextBox
            t.setTitle(noteTitle);
            t.setString(noteContent);
        } catch (InvalidRecordIDException iride) {
            Alert irideAlert = new Alert("ERROR", "Record does not exist. You must "
                                    + "save something before loading it.",
                                    null, AlertType.ERROR);
            irideAlert.setTimeout(Alert.FOREVER);
            Display.getDisplay(this).setCurrent(irideAlert);
        } catch (RecordStoreException rse) {
            Alert rseAlert = new Alert("ERROR", "RecordStoreException", null, 
                                       AlertType.ERROR);
            rseAlert.setTimeout(Alert.FOREVER);
            Display.getDisplay(this).setCurrent(rseAlert);
        } catch (IOException ioe) {
            Alert ioeAlert = new Alert("ERROR", "I/O Exception", null, 
                                        AlertType.ERROR);
            ioeAlert.setTimeout(Alert.FOREVER);
            Display.getDisplay(this).setCurrent(ioeAlert);
        }
    }
    
    public void newLine() {
        noteContent = t.getString();
        
        t.setString(noteContent + "\n");
    }
    
    /*
     * Respond to commands, including exit
     * On the exit command, cleanup and notify that the MIDlet has been destroyed.
     */
    public void commandAction(Command c, Displayable s) {
        if (c == exitCommand) {
            destroyApp(false);
            notifyDestroyed();
        } else if (c == saveCommand) {
            saveNote();
        } else if (c == loadCommand) {
            loadNote();
        } else if (c == newLineCommand) {
            newLine();
        } else if (c == titleOKCommand) {
            saveNote2();
        }
    }
}